@@ -1,5 +1,3 @@
-/* $Id$ */
-
 /*
  *  (C) Copyright 2001-2002 Wojtek Kaniewski <wojtekka@irc.pl>
  *
@@ -24,24 +22,16 @@
  * \brief Obsługa połączeń HTTP
  */
 
-#include <sys/types.h>
-#include <sys/socket.h>
-#include <netinet/in.h>
-#include <arpa/inet.h>
+#include "internal.h"
 
-#include "compat.h"
-#include "libgadu.h"
+#include "strman.h"
+#include "network.h"
 #include "resolver.h"
 
 #include <ctype.h>
 #include <errno.h>
-#include <netdb.h>
-#include <signal.h>
-#include <stdarg.h>
-#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <unistd.h>
 
 #define GG_HTTP_MAX_LENGTH 1000000000
 
@@ -71,7 +61,8 @@
  *
  * \ingroup http
  */
-struct gg_http *gg_http_connect(const char *hostname, int port, int async, const char *method, const char *path, const char *header)
+struct gg_http *gg_http_connect(const char *hostname, int port, int async,
+	const char *method, const char *path, const char *header)
 {
 	struct gg_http *h;
 
@@ -107,7 +98,7 @@ struct gg_http *gg_http_connect(const char *hostname, int port, int async, const
 				method, path, header);
 	}
 
-	if (!h->query) {
+	if (h->query == NULL) {
 		gg_debug(GG_DEBUG_MISC, "// gg_http_connect() not enough memory for query\n");
 		free(h);
 		errno = ENOMEM;
@@ -131,7 +122,7 @@ struct gg_http *gg_http_connect(const char *hostname, int port, int async, const
 		h->timeout = GG_DEFAULT_TIMEOUT;
 	} else {
 		struct in_addr *addr_list = NULL;
-		int addr_count;
+		unsigned int addr_count;
 
 		if (gg_gethostbyname_real(hostname, &addr_list, &addr_count, 0) == -1 || addr_count == 0) {
 			gg_debug(GG_DEBUG_MISC, "// gg_http_connect() host not found\n");
@@ -141,8 +132,12 @@ struct gg_http *gg_http_connect(const char *hostname, int port, int async, const
 			return NULL;
 		}
 
-		if ((h->fd = gg_connect(&addr_list[0], port, 0)) == -1) {
-			gg_debug(GG_DEBUG_MISC, "// gg_http_connect() connection failed (errno=%d, %s)\n", errno, strerror(errno));
+		h->fd = gg_connect(&addr_list[0], port, 0);
+
+		if (h->fd == -1) {
+			gg_debug(GG_DEBUG_MISC, "// gg_http_connect() "
+				"connection failed (errno=%d, %s)\n",
+				errno, strerror(errno));
 			gg_http_free(h);
 			free(addr_list);
 			return NULL;
@@ -173,7 +168,8 @@ struct gg_http *gg_http_connect(const char *hostname, int port, int async, const
 #ifndef DOXYGEN
 
 #define gg_http_error(x) \
-	close(h->fd); \
+	if (h->fd > -1) \
+		close(h->fd); \
 	h->fd = -1; \
 	h->state = GG_STATE_ERROR; \
 	h->error = x; \
@@ -200,18 +196,25 @@ int gg_http_watch_fd(struct gg_http *h)
 {
 	gg_debug(GG_DEBUG_FUNCTION, "** gg_http_watch_fd(%p);\n", h);
 
-	if (!h) {
+	if (h == NULL) {
 		gg_debug(GG_DEBUG_MISC, "// gg_http_watch_fd() invalid arguments\n");
 		errno = EFAULT;
 		return -1;
 	}
 
 	if (h->state == GG_STATE_RESOLVING) {
-		struct in_addr a;
+		struct in_addr addr;
+		int res;
 
 		gg_debug(GG_DEBUG_MISC, "=> http, resolving done\n");
 
-		if (read(h->fd, &a, sizeof(a)) < (signed)sizeof(a) || a.s_addr == INADDR_NONE) {
+		do {
+			res = gg_resolver_recv(h->fd, &addr, sizeof(addr));
+		} while (res == -1 && errno == EINTR);
+
+		h->resolver_cleanup(&h->resolver, 0);
+
+		if (res != sizeof(addr) || addr.s_addr == INADDR_NONE) {
 			gg_debug(GG_DEBUG_MISC, "=> http, resolver thread failed\n");
 			gg_http_error(GG_ERROR_RESOLVING);
 		}
@@ -219,11 +222,11 @@ int gg_http_watch_fd(struct gg_http *h)
 		close(h->fd);
 		h->fd = -1;
 
-		h->resolver_cleanup(&h->resolver, 0);
+		gg_debug(GG_DEBUG_MISC, "=> http, connecting to %s:%d\n", inet_ntoa(addr), h->port);
 
-		gg_debug(GG_DEBUG_MISC, "=> http, connecting to %s:%d\n", inet_ntoa(a), h->port);
+		h->fd = gg_connect(&addr, h->port, h->async);
 
-		if ((h->fd = gg_connect(&a, h->port, h->async)) == -1) {
+		if (h->fd == -1) {
 			gg_debug(GG_DEBUG_MISC, "=> http, connection failed (errno=%d, %s)\n", errno, strerror(errno));
 			gg_http_error(GG_ERROR_CONNECTING);
 		}
@@ -237,10 +240,12 @@ int gg_http_watch_fd(struct gg_http *h)
 
 	if (h->state == GG_STATE_CONNECTING) {
 		int res = 0;
-		unsigned int res_size = sizeof(res);
+		socklen_t res_size = sizeof(res);
 
 		if (h->async && (getsockopt(h->fd, SOL_SOCKET, SO_ERROR, &res, &res_size) || res)) {
-			gg_debug(GG_DEBUG_MISC, "=> http, async connection failed (errno=%d, %s)\n", (res) ? res : errno , strerror((res) ? res : errno));
+			gg_debug(GG_DEBUG_MISC, "=> http, async connection "
+				"failed (errno=%d, %s)\n", (res) ? res : errno,
+				strerror((res) ? res : errno));
 			close(h->fd);
 			h->fd = -1;
 			h->state = GG_STATE_ERROR;
@@ -258,20 +263,34 @@ int gg_http_watch_fd(struct gg_http *h)
 	if (h->state == GG_STATE_SENDING_QUERY) {
 		int res;
 
-		if ((res = write(h->fd, h->query, strlen(h->query))) < 1) {
-			gg_debug(GG_DEBUG_MISC, "=> http, write() failed (len=%d, res=%d, errno=%d)\n", strlen(h->query), res, errno);
+		res = send(h->fd, h->query, strlen(h->query), 0);
+
+		if (res == -1 && errno != EINTR && errno != EAGAIN) {
+			gg_debug(GG_DEBUG_MISC, "=> http, send() failed "
+				"(len=%" GG_SIZE_FMT ", res=%d, errno=%d)\n",
+				strlen(h->query), res, errno);
 			gg_http_error(GG_ERROR_WRITING);
 		}
 
-		if (res < strlen(h->query)) {
-			gg_debug(GG_DEBUG_MISC, "=> http, partial header sent (led=%d, sent=%d)\n", strlen(h->query), res);
+		if (res == -1) {
+			gg_debug(GG_DEBUG_MISC, "=> http, non-critical send "
+				"error (errno=%d, %s)\n",
+				errno, strerror(errno));
+			return 0;
+		}
+
+		if ((size_t) res < strlen(h->query)) {
+			gg_debug(GG_DEBUG_MISC, "=> http, partial header sent "
+				"(led=%" GG_SIZE_FMT ", sent=%d)\n",
+				strlen(h->query), res);
 
 			memmove(h->query, h->query + res, strlen(h->query) - res + 1);
 			h->state = GG_STATE_SENDING_QUERY;
 			h->check = GG_CHECK_WRITE;
 			h->timeout = GG_DEFAULT_TIMEOUT;
 		} else {
-			gg_debug(GG_DEBUG_MISC, "=> http, request sent (len=%d)\n", strlen(h->query));
+			gg_debug(GG_DEBUG_MISC, "=> http, request sent (len=%"
+				GG_SIZE_FMT ")\n", strlen(h->query));
 			free(h->query);
 			h->query = NULL;
 
@@ -287,7 +306,9 @@ int gg_http_watch_fd(struct gg_http *h)
 		char buf[1024], *tmp;
 		int res;
 
-		if ((res = read(h->fd, buf, sizeof(buf))) == -1) {
+		res = recv(h->fd, buf, sizeof(buf), 0);
+
+		if (res == -1 && errno != EINTR && errno != EAGAIN) {
 			gg_debug(GG_DEBUG_MISC, "=> http, reading header failed (errno=%d)\n", errno);
 			if (h->header) {
 				free(h->header);
@@ -296,7 +317,14 @@ int gg_http_watch_fd(struct gg_http *h)
 			gg_http_error(GG_ERROR_READING);
 		}
 
-		if (!res) {
+		if (res == -1) {
+			gg_debug(GG_DEBUG_MISC, "=> http, non-critical recv "
+				"error (errno=%d, %s)\n",
+				errno, strerror(errno));
+			return 0;
+		}
+
+		if (res == 0) {
 			gg_debug(GG_DEBUG_MISC, "=> http, connection reset by peer\n");
 			if (h->header) {
 				free(h->header);
@@ -307,7 +335,9 @@ int gg_http_watch_fd(struct gg_http *h)
 
 		gg_debug(GG_DEBUG_MISC, "=> http, read %d bytes of header\n", res);
 
-		if (!(tmp = realloc(h->header, h->header_size + res + 1))) {
+		tmp = realloc(h->header, h->header_size + res + 1);
+
+		if (tmp == NULL) {
 			gg_debug(GG_DEBUG_MISC, "=> http, not enough memory for header\n");
 			free(h->header);
 			h->header = NULL;
@@ -328,13 +358,18 @@ int gg_http_watch_fd(struct gg_http *h)
 			unsigned int left;
 			char *line;
 
-			left = h->header_size - ((long)(tmp) - (long)(h->header) + sep_len);
+			left = h->header_size - ((size_t)(tmp) - (size_t)(h->header) + sep_len);
 
-			gg_debug(GG_DEBUG_MISC, "=> http, got all header (%d bytes, %d left)\n", h->header_size - left, left);
+			gg_debug(GG_DEBUG_MISC, "=> http, got all header "
+				"(%d bytes, %d left)\n",
+				h->header_size - left, left);
 
 			/* HTTP/1.1 200 OK */
 			if (strlen(h->header) < 16 || strncmp(h->header + 9, "200", 3)) {
-				gg_debug(GG_DEBUG_MISC, "=> -----BEGIN-HTTP-HEADER-----\n%s\n=> -----END-HTTP-HEADER-----\n", h->header);
+				gg_debug(GG_DEBUG_MISC,
+					"=> -----BEGIN-HTTP-HEADER-----\n%s\n"
+					"=> -----END-HTTP-HEADER-----\n",
+					h->header);
 
 				gg_debug(GG_DEBUG_MISC, "=> http, didn't get 200 OK -- no results\n");
 				free(h->header);
@@ -346,7 +381,9 @@ int gg_http_watch_fd(struct gg_http *h)
 			line = h->header;
 			*tmp = 0;
 
-			gg_debug(GG_DEBUG_MISC, "=> -----BEGIN-HTTP-HEADER-----\n%s\n=> -----END-HTTP-HEADER-----\n", h->header);
+			gg_debug(GG_DEBUG_MISC, "=> -----BEGIN-HTTP-HEADER-----"
+				"\n%s\n=> -----END-HTTP-HEADER-----\n",
+				h->header);
 
 			while (line) {
 				if (!strncasecmp(line, "Content-length: ", 16)) {
@@ -368,14 +405,18 @@ int gg_http_watch_fd(struct gg_http *h)
 			}
 
 			if (left > h->body_size) {
-				gg_debug(GG_DEBUG_MISC, "=> http, oversized reply (%d bytes needed, %d bytes left)\n", h->body_size, left);
+				gg_debug(GG_DEBUG_MISC, "=> http, oversized "
+					"reply (%d bytes needed, "
+					"%d bytes left)\n", h->body_size, left);
 				h->body_size = left;
 			}
 
 			gg_debug(GG_DEBUG_MISC, "=> http, body_size=%d\n", h->body_size);
 
 			if (!(h->body = malloc(h->body_size + 1))) {
-				gg_debug(GG_DEBUG_MISC, "=> http, not enough memory (%d bytes for body_buf)\n", h->body_size + 1);
+				gg_debug(GG_DEBUG_MISC, "=> http, not enough "
+					"memory (%d bytes for body_buf)\n",
+					h->body_size + 1);
 				free(h->header);
 				h->header = NULL;
 				gg_http_error(GG_ERROR_READING);
@@ -400,7 +441,9 @@ int gg_http_watch_fd(struct gg_http *h)
 		char buf[1024];
 		int res;
 
-		if ((res = read(h->fd, buf, sizeof(buf))) == -1) {
+		res = recv(h->fd, buf, sizeof(buf), 0);
+
+		if (res == -1 && errno != EINTR && errno != EAGAIN) {
 			gg_debug(GG_DEBUG_MISC, "=> http, reading body failed (errno=%d)\n", errno);
 			if (h->body) {
 				free(h->body);
@@ -409,14 +452,24 @@ int gg_http_watch_fd(struct gg_http *h)
 			gg_http_error(GG_ERROR_READING);
 		}
 
-		if (!res) {
+		if (res == -1) {
+			gg_debug(GG_DEBUG_MISC, "=> http, non-critical "
+				"recv error (errno=%d, %s)\n",
+				errno, strerror(errno));
+			return 0;
+		}
+
+		if (res == 0) {
 			if (h->body_done >= h->body_size) {
 				gg_debug(GG_DEBUG_MISC, "=> http, we're done, closing socket\n");
 				h->state = GG_STATE_PARSING;
 				close(h->fd);
 				h->fd = -1;
 			} else {
-				gg_debug(GG_DEBUG_MISC, "=> http, connection closed while reading (have %d, need %d)\n", h->body_done, h->body_size);
+				gg_debug(GG_DEBUG_MISC, "=> http, "
+					"connection closed while reading "
+					"(have %d, need %d)\n",
+					h->body_done, h->body_size);
 				if (h->body) {
 					free(h->body);
 					h->body = NULL;
@@ -432,10 +485,14 @@ int gg_http_watch_fd(struct gg_http *h)
 		if (h->body_done + res > h->body_size) {
 			char *tmp;
 
-			gg_debug(GG_DEBUG_MISC, "=> http, too much data (%d bytes, %d needed), enlarging buffer\n", h->body_done + res, h->body_size);
+			gg_debug(GG_DEBUG_MISC, "=> http, too much data "
+				"(%d bytes, %d needed), enlarging buffer\n",
+				h->body_done + res, h->body_size);
 
 			if (!(tmp = realloc(h->body, h->body_done + res + 1))) {
-				gg_debug(GG_DEBUG_MISC, "=> http, not enough memory for data (%d needed)\n", h->body_done + res + 1);
+				gg_debug(GG_DEBUG_MISC, "=> http, not enough "
+					"memory for data (%d needed)\n",
+					h->body_done + res + 1);
 				free(h->body);
 				h->body = NULL;
 				gg_http_error(GG_ERROR_READING);
@@ -481,12 +538,12 @@ void gg_http_stop(struct gg_http *h)
 	if (h->state == GG_STATE_ERROR || h->state == GG_STATE_DONE)
 		return;
 
+	h->resolver_cleanup(&h->resolver, 1);
+
 	if (h->fd != -1) {
 		close(h->fd);
 		h->fd = -1;
 	}
-
-	h->resolver_cleanup(&h->resolver, 1);
 }
 
 /**
@@ -498,23 +555,17 @@ void gg_http_stop(struct gg_http *h)
  */
 void gg_http_free_fields(struct gg_http *h)
 {
-	if (!h)
+	if (h == NULL)
 		return;
 
-	if (h->body) {
-		free(h->body);
-		h->body = NULL;
-	}
+	free(h->body);
+	h->body = NULL;
 
-	if (h->query) {
-		free(h->query);
-		h->query = NULL;
-	}
+	free(h->query);
+	h->query = NULL;
 
-	if (h->header) {
-		free(h->header);
-		h->header = NULL;
-	}
+	free(h->header);
+	h->header = NULL;
 }
 
 /**
@@ -528,7 +579,7 @@ void gg_http_free_fields(struct gg_http *h)
  */
 void gg_http_free(struct gg_http *h)
 {
-	if (!h)
+	if (h == NULL)
 		return;
 
 	gg_http_stop(h);
