@@ -16,6 +16,8 @@
  *  USA.
  */
 
+#include "internal.h"
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -26,17 +28,17 @@
 #include <string.h>
 #include <errno.h>
 #include <ctype.h>
-#include "config.h"
 
 #if defined(GG_CONFIG_HAVE_PTHREAD)
 #  include <pthread.h>
 #elif defined(_WIN32)
 #  define GG_SIMULATE_WIN32_PTHREAD
+#else
+/* pthreads are required to run server thread */
+#  error "pthreads missing"
 #endif
 
-#include "libgadu.h"
 #include "network.h"
-#include "internal.h"
 
 #ifdef GG_CONFIG_HAVE_GNUTLS
 #include <gnutls/gnutls.h>
@@ -298,7 +300,7 @@ static int h_errno;
 
 #undef gethostbyname
 #ifdef _WIN32
-static struct hostent *my_gethostbyname(const char *name)
+static struct hostent * WSAAPI my_gethostbyname(const char *name)
 #else
 struct hostent *gethostbyname(const char *name)
 #endif
@@ -375,7 +377,7 @@ int gethostbyname_r(const char *name, struct hostent *ret, char *buf,
 #ifdef _WIN32
 static gg_win32_hook_data_t connect_hook;
 
-static int my_connect(SOCKET socket, const struct sockaddr *address, int address_len)
+static int WSAAPI my_connect(SOCKET socket, const struct sockaddr *address, int address_len)
 #else
 int connect(int socket, const struct sockaddr *address, socklen_t address_len)
 #endif
@@ -536,7 +538,7 @@ int connect(int socket, const struct sockaddr *address, socklen_t address_len)
 #ifdef _WIN32
 static gg_win32_hook_data_t get_last_error_hook;
 
-static int my_get_last_error(void)
+static int WSAAPI my_get_last_error(void)
 {
 	int result;
 
